import warnings

import hvac


def hashivault_init(readonly=True):
    argument_spec = dict(
        url = dict(required=False, default=os.environ.get('VAULT_ADDR', ''), type='str'),
        verify = dict(required=False, default=(not os.environ.get('VAULT_SKIP_VERIFY', '')), type='bool'),
        authtype = dict(required=False, default='token', type='str'),
        token = dict(required=False, default=os.environ.get('VAULT_TOKEN', ''), type='str'),
        username = dict(required=False, type='str'),
        password = dict(required=False, type='str'),
        secret = dict(required=True, type='str'),
    )
    if readonly:
        argument_spec['key'] = dict(required=True, type='str')
        argument_spec['register'] = dict(required=False, type='str')
    else:
        argument_spec['update'] = dict(required=False, default=False, type='bool')
        argument_spec['data'] = dict(required=False, default={}, type='dict')
    return AnsibleModule(argument_spec=argument_spec)


def hashivault_client(params):
    url = params.get('url')
    verify = params.get('verify')
    token = params.get('token')
    authtype = params.get('authtype')
    token = params.get('token')
    username = params.get('username')
    password = params.get('password')
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        client = hvac.Client(url=url, verify=verify)
        if authtype == 'github':
            client.auth_github(token)
        elif authtype == 'userpass':
            client.auth_userpass(username, password)
        elif authtype == 'ldap':
            client.auth_ldap(username, password)
        else:
            client.token = token
    return client


def hashivault_read(params):
    result = { "changed": False, "rc" : 0}
    try:
        client = hashivault_client(params)
        secret = params.get('secret')
        key = params.get('key')
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            data = client.read('secret/%s' % secret)['data']
        value = data[key]
        result['value'] = value
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = "Exception: " + str(e)
        result['stack_trace'] = traceback.format_exc()
    return result


def hashivault_write(params):
    result = { "changed": False, "rc" : 0}
    try:
        client = hashivault_client(params)
        secret = params.get('secret')
        data = params.get('data')
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            if params.get('update'):
                write_data = client.read('secret/%s' % secret)
                write_data  = write_data['data']
                write_data.update(data)
                client.write(('secret/%s' % secret), **write_data)
                result['msg'] = "Secret %s updated" % secret
            else:
                client.write(('secret/%s' % secret), **data)
                result['msg'] = "Secret %s written" % secret
        result['changed'] = True
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = "Exception: " + str(e)
        result['stack_trace'] = traceback.format_exc()
    return result



#!/usr/bin/env python
DOCUMENTATION = '''
---
module: hashivault_read
version_added: "0.1"
short_description: Hashicorp Vault read module
description:
    - Module to read to Hashicorp Vault.
options:
    url:
        description:
            - url for vault
        default: to environment variable VAULT_ADDR
    verify:
        description:
            - verify TLS certificate
        default: to environment variable VAULT_SKIP_VERIFY
    authtype:
        description:
            - authentication type to use: token, userpass, github, ldap
        default: token
    token:
        description:
            - token for vault
        default: to environment variable VAULT_TOKEN
    username:
        description:
            - username to login to vault.
        default: False
    password:
        description:
            - password to login to vault.
        default: False
    secret:
        description:
            - secret to read.
        default: False
    key:
        description:
            - secret key to read.
        default: False
    register:
        description:
            - variable to register result.
        default: False
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_read: secret='giant' key='fie' register='fie'
    - debug: msg="Value is {{fie.value}}"
'''


def main():
    module = hashivault_init()
    result = hashivault_read(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


from ansible.module_utils.basic import *
from ansible.module_utils.hashivault import *

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from distutils.core import setup

files = [
    "ansible/module_utils",
    "ansible/modules/hashivault",
]

setup(name='ansible-modules-hashivault',
      version='0.9',
      description='Ansible Modules for Hashicorp Vault',
      author='Terry Howe',
      author_email='terrylhowe@example.com',
      url='https://github.com/TerryHowe/ansible-modules-hashivault',
      packages=files,
)
